'use strict';
'use struct';

function HashMap() {
    /** Map 大小 **/
    var size = 0;
    /** 对象 **/
    var entry = new Object();

    /** 存 **/
    this.put = function (key, value) {
        if (!this.containsKey(key)) {
            size++;
        }
        entry[key] = value;
    }

    /** 取 **/
    this.get = function (key) {
        if (this.containsKey(key)) {
            return entry[key];
        }
        else {
            return null;
        }
    }

    /** 删除 **/
    this.remove = function (key) {
        if (delete entry[key]) {
            size--;
        }
    }

    /** 是否包含 Key **/
    this.containsKey = function (key) {
        return (key in entry);
    }

    /** 是否包含 Value **/
    this.containsValue = function (value) {
        for (var prop in entry) {
            if (entry[prop] == value) {
                return true;
            }
        }
        return false;
    }

    /** 所有 Value **/
    this.values = function () {
        var values = new Array(size);
        for (var prop in entry) {
            values.push(entry[prop]);
        }
        return values;
    }

    /** 所有 Key **/
    this.keys = function () {
        var keys = new Array(size);
        for (var prop in entry) {
            keys.push(prop);
        }
        return keys;
    }

    /** Map Size **/
    this.size = function () {
        return size;
    }
}

var RE = {
    currentRange: {
        startContainer: null,
        startOffset: 0,
        endContainer: null,
        endOffset: 0
    },
    cache: {
        editor: null,
        title: null,
        currentLink: null,
        line: null
    },
    commandSet: ['bold', 'italic', 'strikethrough', 'redo', 'undo'],
    schemeCache: {
        FOCUS_SCHEME: 'focus://',
        CHANGE_SCHEME: 'change://',
        STATE_SCHEME: 're-state://',
        CALLBACK_SCHEME: 're-callback://',
        IMAGE_SCHEME: 're-image://'
    },
    setting: {
        screenWidth: 0,
        margin: 20
    },
    imageCache: new HashMap(),
    init: function init() {
        //初始化内部变量
        var _self = this;
        _self.initCache();
        _self.initSetting();
        _self.bind();
        _self.focus();
    },
    bind: function bind() {
        var _self = this;

        var _self$schemeCache = _self.schemeCache,
            FOCUS_SCHEME = _self$schemeCache.FOCUS_SCHEME,
            STATE_SCHEME = _self$schemeCache.STATE_SCHEME,
            CALLBACK_SCHEME = _self$schemeCache.CALLBACK_SCHEME;


        document.addEventListener('selectionchange', _self.saveRange, false);

        _self.cache.title.addEventListener('focus', function (evt) {
            RichEditorInterface.onTitleFocusState(true);
        }, false);

        _self.cache.title.addEventListener('blur', function (evt) {
            RichEditorInterface.onTitleFocusState(false);
        }, false);

        //获取焦点
        _self.cache.editor.addEventListener('focus', function (evt) {
            // 输入时获取html的内容，length = 0添加p标签
            if (_self.cache.editor.innerHTML.length == 0) {
                _self.cache.editor.innerHTML = "<p><br></p>"
            }
            RichEditorInterface.onEditorFocusState(true);
            _self.getEditItem(evt);
        }, false);

        //失去焦点
        _self.cache.editor.addEventListener('blur', function (evt) {
//            // 如果只有"<p><br></p>"则作为无文本处理
            if (_self.cache.editor.innerHTML == "<p><br></p>") {
                _self.cache.editor.innerHTML = ""
            }
            _self.saveRange();
            RichEditorInterface.onEditorFocusState(false);
        }, false);

        _self.cache.editor.addEventListener('click', function (evt) {
            _self.saveRange();
            _self.getEditItem(evt);
        }, false);

        _self.cache.editor.addEventListener('keyup', function (evt) {
//            // 输入时获取html的内容，length = 0添加p标签
            if (_self.cache.editor.innerHTML.length == 0) {
                _self.cache.editor.innerHTML = "<p><br></p>"
            }
            if (evt.which == 37 || evt.which == 39 || evt.which == 13 || evt.which == 8) {
                _self.getEditItem(evt);
            }
        }, false);

        _self.cache.editor.addEventListener('input', function () {
            RichEditorInterface.onEditorContentStartWords(_self.staticWords());
        }, false);
        _self.cache.editor.addEventListener('copy', function (evt) {
            //阻止默认复制内容
            evt.preventDefault();
            //拿到剪切板
            var clipboardData = evt.clipboardData;
            //获得已选择的文字
            var content = window.getSelection().toString();
            //设置剪切板中内容
            clipboardData.setData("text",content);
            }, false);
    },
    /**
     * 设置当前浏览器的最小高度
     */
    initCache: function initCache() {
        var _self = this;
        _self.cache.editor = document.getElementById('editor');
        _self.cache.title = document.getElementById('title');
        _self.cache.line = document.getElementsByClassName('line')[0];
        _self.cache.editor.style.minHeight = window.innerHeight - 69 + 'px';
    },
    initSetting: function initSetting() {
        var _self = this;
        _self.setting.screenWidth = window.screen.width - 20;//屏幕分辨率的宽
    },
    blurFocus: function blurFocus() {
        var _self = this;
        _self.cache.editor.blur();
    },
    focus: function focus() {
        //聚焦
        var _self = this;
        var range = document.createRange();
        range.selectNodeContents(this.cache.editor);
        range.collapse(false);
        var select = window.getSelection();
        select.removeAllRanges();
        select.addRange(range);
        _self.cache.editor.focus();
    },

    getHtml: function getHtml() {
        var _self = this;
        var content = _self.cache.editor.innerHTML;
        content = content.replace(/^\s*(<p>(\s|<br\/?>)*<\/p>\s*)*/i, '').replace(/\s*(<p>(\s|<br\/?>)*<\/p>\s*)*$/i, '');
        content = content.replace(/&nbsp;/g, ' ').trim().replace(/&amp;/g, '&');
        return content;
    },
    getHtmlContentLength: function getHtmlContentLength() {
        var _self = this;
        RichEditorInterface.onEditorContentStartWords(_self.staticWords());
    },
    staticWords: function staticWords() {
        var _self = this;
        var content = _self.getHtml().replace(/<div\sclass="tips">.*<\/div>|<\/?[^>]*>/g, '').replace(/\s+/, '').trim();
        return content.length;
    },
    saveRange: function saveRange() {
        //保存节点位置
        var _self = this;
        var selection = window.getSelection();
        if (selection.rangeCount > 0) {
            var range = selection.getRangeAt(0);
            var startContainer = range.startContainer,
                startOffset = range.startOffset,
                endContainer = range.endContainer,
                endOffset = range.endOffset;
            _self.currentRange = {
                startContainer: startContainer,
                startOffset: startOffset,
                endContainer: endContainer,
                endOffset: endOffset
            };
        }
    },
    reduceRange: function reduceRange() {
        //还原节点位置
        var _self = this;
        var _self$currentRange = _self.currentRange,
            startContainer = _self$currentRange.startContainer,
            startOffset = _self$currentRange.startOffset,
            endContainer = _self$currentRange.endContainer,
            endOffset = _self$currentRange.endOffset;

        var range = document.createRange();
        var selection = window.getSelection();
        selection.removeAllRanges();
        range.setStart(startContainer, startOffset);
        range.setEnd(endContainer, endOffset);
        selection.addRange(range);
    },
    exec: function exec(command) {
        //执行指令
        var _self = this;
        if (_self.commandSet.indexOf(command) !== -1) {
            document.execCommand(command, false, null);
        } else {
            var value = '<' + command + '>';
            document.execCommand('formatBlock', false, value);
            _self.getEditItem({});
        }
    },
    getEditItem: function getEditItem(evt) {
        //通过点击时，去获得一个当前位置的所有状态
        var _self = this;
        var _self$schemeCache2 = _self.schemeCache,
            STATE_SCHEME = _self$schemeCache2.STATE_SCHEME,
            CHANGE_SCHEME = _self$schemeCache2.CHANGE_SCHEME,
            IMAGE_SCHEME = _self$schemeCache2.IMAGE_SCHEME;

        if (evt.target && evt.target.tagName === 'A') {
            _self.cache.currentLink = evt.target;
            var name = evt.target.innerText;
            var href = evt.target.getAttribute('href');
            window.location.href = CHANGE_SCHEME + encodeURI(name + '@_@' + href);
        } else {
            if (evt.which == 8) {
                _self.getImageData();
                RichEditorInterface.onEditorContentStartWords(_self.staticWords());
            }
            var items = [];
            _self.commandSet.forEach(function (item) {
                if (document.queryCommandState(item)) {
                    items.push(item);
                }
            });
            if (document.queryCommandValue('formatBlock')) {
                items.push(document.queryCommandValue('formatBlock'));
            }
            window.location.href = STATE_SCHEME + encodeURI(items.join(','));
        }
    },
    setPadding: function setPadding(left, top, right, bottom) {
        var _self = this;
        _self.cache.editor.style.paddingLeft = left;
        _self.cache.editor.style.paddingTop = top;
        _self.cache.editor.style.paddingRight = right;
        _self.cache.editor.style.paddingBottom = bottom;

        _self.cache.title.style.paddingLeft = left;
        _self.cache.title.style.paddingTop = top;
        _self.cache.title.style.paddingRight = right;
        _self.cache.title.style.paddingBottom = bottom;
    },
    insertHtml: function insertHtml(html) {
        var _self = this;
        document.execCommand('insertHtml', false, html);
    },

    /**
     * 获取image数量及内容
     */
    getImageData: function getImageData() {
        var objs = document.getElementsByTagName("img");
        var imageIds = "";
        for (var i = 0; i < objs.length; i++) {
            var obj = objs[i];
            var imageId = obj.getAttribute("data-id");
            if (imageId == null) {
                continue;
            }
            imageIds = imageIds.concat(imageId + ",");
        }
        RichEditorInterface.onGetImageData(imageIds);
    },
    /**
     * 回显示
     */
    setHtmlContent: function setHtmlContent(content) {
        var _self = this;
        if (content.length > 0 && content != null) {
            _self.cache.editor.innerHTML = decodeURIComponent(content.replace(/\+/g, '%20')) + "<p><br></p>";
        } else {
            _self.cache.editor.innerHTML = decodeURIComponent(content.replace(/\+/g, '%20'))
        }
        //获取所有的imageId并设置点击事件
        var objs = document.getElementsByTagName("img");
        for (var i = 0; i < objs.length; i++) {
            var obj = objs[i];
            var imageId = obj.getAttribute("data-id");
            if (imageId != null) {
                var img = document.querySelector('img[data-id="' + imageId + '"]');
                if (img != null) {
                    var imgBlock = img.parentNode;
                    if (imgBlock != null) {
                        imgBlock.addEventListener('click', function (e) {
                            e.stopPropagation();
                            _self.blurFocus();
                            var current = e.currentTarget;
                            var img = current.querySelector('.images');
                            var id = img.getAttribute('data-id');
                            window.location.href = _self.schemeCache.IMAGE_SCHEME + encodeURI(id);
                        }, false);
                        _self.imageCache.put(imageId, imgBlock.parentNode);
                    }
                }
            }
        }
    },

    /**
     * 回显示
     */
    setTitleContent: function setTitleContent(title) {
        var _self = this;
        _self.cache.title.innerHTML = decodeURIComponent(title.replace(/\+/g, '%20'));
    },

    /**
     * 获取文本信息
     */
    getHtmlContent: function getHtmlContent() {
        var _self = this;
        var title = _self.cache.title.innerHTML.trim();
        title = title.replace(/&nbsp;/g, ' ').trim().replace(/  /g, '&nbsp; ').trim().replace(/&amp;/g, '&');
        title = _self.htmlDecodeByRegExp(title);
        var content = _self.getHtml();
        RichEditorInterface.getTitleAndContent(title, content);
    },

    setBackgroundColor: function setBackgroundColor(color) {
        var _self = this;
        document.body.style.backgroundColor = color;
    },
    setFontColor: function setFontColor(color) {
        document.body.style.color = color;
    },
    setLineColor: function setLineColor(color) {
        var _self = this;
        _self.cache.editor.style.borderColor = color;
    },
    setTitlePlaceholder: function setTitlePlaceholder(title_placeholder) {
        var _self = this;
        _self.cache.title.setAttribute("title_placeholder", title_placeholder);
    },
    setEditorPlaceholder: function setEditorPlaceholder(editor_placeholder) {
        var _self = this;
        _self.cache.editor.setAttribute("editor_placeholder", editor_placeholder);
    },
    insertLine: function insertLine() {
        var _self = this;
        var html = '<hr><p><br></p>';
        _self.insertHtml(html);
        _self.getEditItem({});
    },
    insertLink: function insertLink(name, url) {
        var _self = this;
        var html = '<a href="' + url + '" class="editor-link">' + name + '</a>';
        _self.insertHtml(html);
    },
    changeLink: function changeLink(name, url) {
        var _self = this;
        var current = _self.cache.currentLink;
        var len = name.length;
        current.innerText = name;
        current.setAttribute('href', url);
        var selection = window.getSelection();
        var range = selection.getRangeAt(0).cloneRange();
        var _self$currentRange2 = _self.currentRange,
            startContainer = _self$currentRange2.startContainer,
            endContainer = _self$currentRange2.endContainer;

        selection.removeAllRanges();
        range.setStart(startContainer, len);
        range.setEnd(endContainer, len);
        selection.addRange(range);
    },
    //添加图片
    insertImage: function insertImage(url, id, width, height) {
        var _self = this;
        var newWidth = 0,
            newHeight = 0;
        var screenWidth = _self.setting.screenWidth;
        if (width > screenWidth) {
            newWidth = screenWidth;
            newHeight = height * newWidth / width;
        } else {
            newWidth = width;
            newHeight = height;
        }
        var image =
            '<div class="block">' +
            '<div class="img-block">' +
            '<div style="width: ' + newWidth + 'px" class="process">' +
            '<div class="fill"></div>' +
            '</div>' +
            '<img class="images" data-id="' + id + '" style="width: ' + newWidth + 'px; height: ' + newHeight + 'px;" src="' + url + '"/>' +
            '<div class="cover" style="width: ' + newWidth + 'px; height: ' + newHeight + 'px"></div>' +
            '<div class="delete">' +
            '<img src="./reload.png">' +
            '</div>' +
            '</div>' +
            '</div>' +
            '<p><br></p>';
        _self.insertHtml(image);
        console.log("_self.getHtml() = " + _self.getHtml());
        ;
        var img = document.querySelector('img[data-id="' + id + '"]');
        var imgBlock = img.parentNode;
        imgBlock.parentNode.contentEditable = false;
        imgBlock.addEventListener('click', function (e) {
            e.stopPropagation();
            _self.blurFocus();
            var current = e.currentTarget;
            var img = current.querySelector('.images');
            var id = img.getAttribute('data-id');
            window.location.href = _self.schemeCache.IMAGE_SCHEME + encodeURI(id);
        }, false);
        _self.imageCache.put(id, imgBlock.parentNode);
    },

    //更新图片上传进度
    changeProcess: function changeProcess(id, process) {
        var _self = this;
        var block = _self.imageCache.get(id);
        var fill = block.querySelector('.fill');
        fill.style.width = process + '%';
        if (process == 100) {
            var cover = block.querySelector('.cover');
            var process = block.querySelector('.process');
            var imgBlock = block.querySelector('.img-block');
            imgBlock.removeChild(cover);
            imgBlock.removeChild(process);
        }
    },

    //替换图片为网络地址
    replaceImageUrl: function replaceImageUrl(url, id) {
        var _self = this;
        var img = document.querySelector('img[data-id="' + id + '"]');
        img.setAttribute('src', url);
        //删除delete标签
        var block = _self.imageCache.get(id);
        var imgBlock = block.querySelector('.img-block');
        var del = block.querySelector('.delete');
        imgBlock.removeChild(del);
    },

    //删除图片
    removeImage: function removeImage(id) {
        var _self = this;
        var img = document.querySelector('img[data-id="' + id + '"]');
        var block = img.parentNode.parentNode;
        block.parentNode.removeChild(block);
        _self.imageCache.remove(id);
    },

    //图片上传失败
    uploadFailure: function uploadFailure(id) {
        var _self = this;
        var block = _self.imageCache.get(id);
        var del = block.querySelector('.delete');
        del.style.display = 'block';
    },

    //图片重新加载
    uploadReload: function uploadReload(id) {
        var _self = this;
        var block = _self.imageCache.get(id);
        var del = block.querySelector('.delete');
        del.style.display = 'none';
    },

    htmlDecodeByRegExp: function htmlDecodeByRegExp(str) {
        var s = "";
        if (str.length == 0) return "";
        s = str.replace(/&amp;/g, "&");
        s = s.replace(/&lt;/g, "<");
        s = s.replace(/&gt;/g, ">");
        s = s.replace(/&nbsp;/g, " ");
        s = s.replace(/&#39;/g, "\'");
        s = s.replace(/&quot;/g, "\"");
        return s;
    }

};
RE.init();